package cs108;

import java.util.Iterator;
import java.util.List;
import java.util.function.Consumer;

public class PrintAll {
    // v1: itérateur explicite
    public static <T> void printAll1(Iterable<T> i) {
        Iterator<T> it = i.iterator();
        while (it.hasNext())
            System.out.println(it.next());
    }

    // v2: boucle for-each (itérateur implicite)
    public static <T> void printAll2(Iterable<T> i) {
        for (T t: i)
            System.out.println(t);
    }

    // v3: méthode forEach sans lambda
    public static <T> void printAll3(Iterable<T> i) {
        i.forEach(new Printer<T>());
    }

    public static class Printer<T> implements Consumer<T> {
        @Override
        public void accept(T x) {
            System.out.println(x);                
        }
    }

    // v4: méthode forEach avec lambda
    public static <T> void printAll4(Iterable<T> i) {
        i.forEach(x -> System.out.println(x));
    }

    // v5: méthode forEach avec référence de méthode
    public static <T> void printAll5(Iterable<T> i) {
        i.forEach(System.out::println);
    }

    public static void main(String[] args) {
        List<String> l = List.of("un", "deux", "trois");
        printAll1(l);
        printAll2(l);
        printAll3(l);
        printAll4(l);
        printAll5(l);
    }
}
