package cs108;

import java.util.Collection;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

public class WordLength {
    // v1: en utilisant uniquement les méthodes "simples" de Map
    public static Map<Integer, Set<String>> groupByLen1(Collection<String> c) {
        Map<Integer, Set<String>> m = new TreeMap<>();
        for (String s: c) {
            int l = s.length();
            if (! m.containsKey(l))
                m.put(l, new TreeSet<>());
            m.get(l).add(s);
        }
        return m;
    }

    // v2: en utilisant les méthodes "sophistiquées" de Map (utilisant des lambdas)
    public static Map<Integer, Set<String>> groupByLen2(Collection<String> c) {
        Map<Integer, Set<String>> m = new TreeMap<>();
        for (String s: c)
            m.computeIfAbsent(s.length(), k -> new TreeSet<>()).add(s);
        return m;
    }

    // v1: en utilisant une itération sur la vue sur les paires clef/valeur
    public static void printWords1(Map<Integer, Set<String>> m) {
        for (Map.Entry<Integer, Set<String>> e: m.entrySet()) {
            System.out.printf("Mots de longueur %d:%n", e.getKey());
            for (String w: e.getValue())
                System.out.printf("  %s%n", w);
        }
    }
    
    // v2: en utilisant les méthodes forEach de Map et Set.
    public static void printWords2(Map<Integer, Set<String>> m) {
        m.forEach((l, w) -> {
            System.out.printf("Mots de longueur %d:%n", l);
            w.forEach(s -> System.out.printf("  %s%n", s));
        });
    }
    
    public static void main(String[] args) {
        Set<String> w = Set.of("un", "deux", "trois", "quatre", "cinq");
        printWords1(groupByLen1(w));
        System.out.println();
        printWords2(groupByLen2(w));
    }
}
